@@ -70,6 +70,7 @@ variables_new(const char *name)
 variable *
 variables_new_with_widget(
 		AttributeSet *Attr, 
+		tag_attr     *widget_tag_attr,
 		GtkWidget    *widget, 
 		int           type)
 {
@@ -110,6 +111,7 @@ variables_new_with_widget(
 	var->Type = type;
 	var->autonamed = autonamed;
 	var->Attributes = Attr;
+	var->widget_tag_attr = widget_tag_attr;
 #ifdef DEBUG
 	g_message("%s(): End.", __func__);
 #endif
@@ -305,6 +307,9 @@ variables_refresh(const char *name)
 		case WIDGET_CHECKBOX:
 			widget_checkbox_refresh(var);
 			break;
+		case WIDGET_TREE:
+			widget_tree_refresh(var);
+			break;
 #ifdef DEBUG
 		default:
 			if (!option_no_warning)
@@ -532,7 +537,7 @@ find_variable_by_widget(GtkWidget *widget)
 
 
 static void 
-_variables_initialize(variable * actual)
+_variables_initialize(variable *actual)
 {
 	char *socket_id;
 	char command[128];
@@ -647,9 +652,9 @@ _variables_export(variable *actual)
 			 */
 			tmp = g_object_get_data(G_OBJECT(actual->Widget), "exported_column");
 			if (tmp != 0)
-				column = atoi(tmp);
+				column = atoi(tmp) + FirstDataColumn;
 			else
-				column = 0;
+				column = FirstDataColumn;
 			
 			line = g_strdup_printf("%s_ALL=\"", actual->Name);
 			n = 0;
@@ -747,9 +752,9 @@ print_variables(variable * actual)
 					printf("'%s'", gtk_object_get_user_data (itemlist->data));
 				else
 					printf(" '%s'", gtk_object_get_user_data (itemlist->data));
-				++n;
 next_item:
 				itemlist = itemlist->next;
+				++n;
 			}
 			printf("\"\n");
 		}
@@ -762,10 +767,13 @@ next_item:
 			 * Which column should we print?
 			 */
 			tmp = g_object_get_data(G_OBJECT(actual->Widget), "exported_column");
-			if (tmp != 0)
-				column = atoi(tmp);
-			else
-				column = 0;
+			if (tmp != 0) {
+				column = atoi(tmp) + FirstDataColumn;
+				//g_message("%s: Exported column: %d", __func__, column);
+				//g_message("%s: FirstDataColumn: %d", __func__, FirstDataColumn);
+			} else {
+				column = FirstDataColumn;
+			}
 			
 			line = g_strdup_printf("%s_ALL=\"", actual->Name);
 			n = 0;
@@ -781,6 +789,7 @@ next_item:
                                 
 				if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter))
 					break;
+				++n;
 			}
 			tmp = g_strconcat(line, "\"\n", NULL);
 			g_printf("%s", tmp);
