@@ -18,9 +18,14 @@
 **
  */
 
+#include <gtk/gtk.h>
 #include "widgets.h"
 #include "stringman.h"
 
+#undef DEBUG
+#undef WARNING
+#include "macros.h"
+
 extern gboolean option_no_warning;
 
 static void 
@@ -178,9 +183,9 @@ widget_get_text_value(
 				 */
 				tmp = g_object_get_data(G_OBJECT(widget), "exported_column");
 				if (tmp != 0)
-					n = atoi(tmp);
+					n = atoi(tmp) + FirstDataColumn;
 				else
-					n = 0;
+					n = FirstDataColumn;
 				/*
 				 * Returning the text from the selected row.
 				 */
@@ -456,8 +461,222 @@ void widget_list_refresh(variable * var)
 	gtk_widget_queue_draw(var->Widget);
 }
 
+/******************************************************************************
+ * Handling TreeView widgets. The model always has the following columns:     *
+ *   0) "pixbuf"     GdkPixbuf                                                *
+ *   1) "icon-name"  gchararray                                               *
+ *   2) "stock-id"   gchararray                                               *
+ *                                                                            *
+ * We create the TreeStore in the following function:                         *
+ *   gtkdialog_tree_store_new                                                 *
+ * We create the TreeView in:                                                 *
+ *   create_tree_view                                                         *
+ *                                                                            *
+ *                                                                            *
+ ******************************************************************************/
+void
+fill_tree_model_by_items(
+	GtkTreeModel  *tree_model,
+	variable      *var)
+{
+	gchar        *act;
+	gchar       **columns;
+	gint          n, ncolumns;
+	GType         coltype;
+	GtkTreeIter   iter;
+	gchar        *stock_name;
+	gchar        *tagattr;
 
-void widget_table_refresh(variable * var)
+	PIP_DEBUG("tree_model: %p var: %p", tree_model, var);
+	
+	ncolumns = gtk_tree_model_get_n_columns(tree_model) - FirstDataColumn;
+	
+	act = attributeset_get_first(var->Attributes, ATTR_ITEM);
+	while (act != NULL) {
+		gtk_tree_store_append(GTK_TREE_STORE(tree_model), &iter, NULL);
+		
+		tagattr = attributeset_get_this_tagattr(var->Attributes, ATTR_ITEM, "stock");
+		if (tagattr != NULL)
+			gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter,
+					ColumnStockId, tagattr,
+					-1);
+
+		tagattr = attributeset_get_this_tagattr(var->Attributes, ATTR_ITEM, "stock_id");
+		if (tagattr != NULL)
+			gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter,
+					ColumnStockId, tagattr,
+					-1);
+		
+		tagattr = attributeset_get_this_tagattr(var->Attributes, ATTR_ITEM, "icon");
+		if (tagattr != NULL)
+			gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter,
+					ColumnIconName, tagattr,
+					-1);
+		
+		tagattr = attributeset_get_this_tagattr(var->Attributes, ATTR_ITEM, "icon_name");
+		if (tagattr != NULL)
+			gtk_tree_store_set(GTK_TREE_STORE(tree_model), &iter,
+					ColumnIconName, tagattr,
+					-1);
+
+		columns = g_strsplit(act, "|", 128);
+		for (n = 0; columns[n] != NULL; ++n) {
+			if (n >= ncolumns)
+				break;
+			g_strstrip(columns[n]);
+			coltype = gtk_tree_model_get_column_type(tree_model, n + FirstDataColumn);
+			
+			switch (coltype) {
+				case G_TYPE_STRING:
+					gtk_tree_store_set(
+						GTK_TREE_STORE(tree_model),
+						&iter,
+						n + FirstDataColumn, columns[n],
+						-1);
+					break;
+				default:
+					g_warning("%s(): Unhandled column type", __func__);
+			}
+		}
+		g_strfreev(columns);
+
+next_item:
+		act = attributeset_get_next(var->Attributes, ATTR_ITEM);
+	}
+}
+
+static void
+fill_tree_view_by_command(
+		GtkWidget     *tree_view, 
+		GtkTreeModel  *tree_model, 
+		const gchar   *command, 
+		gint           stock_column, 
+		gint           icon_column)
+{
+	FILE      *infile;
+	char        oneline[512];
+	gchar     **columns;
+	gint        n, ncolumns;
+	GType       coltype;
+	GtkTreeIter iter;
+	gint        hiddencolumns;
+
+
+	PIP_DEBUG("tree_view: %p, command: '%s'", tree_view, command);
+
+	infile = widget_opencommand(command);
+	if (infile == NULL) {
+		g_warning("%s(): command %s, %m", __func__, command);
+		return;
+	}
+
+	
+	ncolumns = gtk_tree_model_get_n_columns(tree_model) - FirstDataColumn;
+	while (fgets(oneline, 512, infile) != NULL) {
+		PIP_DEBUG("Line from pipe: '%s'", oneline);
+
+		hiddencolumns = 0;
+		g_strstrip(oneline);
+		columns = g_strsplit(oneline, "|", 128);
+		gtk_tree_store_append(GTK_TREE_STORE(tree_model), &iter, NULL);
+		for (n = 0; columns[n] != NULL; ++n) {
+			if (n - hiddencolumns >= ncolumns)
+				break;
+			g_strstrip(columns[n]);
+			if (n == stock_column) {
+				PIP_DEBUG("Stock: '%s'", columns[n]);
+				gtk_tree_store_set(
+					GTK_TREE_STORE(tree_model), &iter,
+					ColumnStockId, columns[n],
+						-1);
+				++hiddencolumns;
+				continue;
+			}
+			
+			if (n == icon_column) {
+				PIP_DEBUG("Icon: '%s'", columns[n]);
+				gtk_tree_store_set(
+					GTK_TREE_STORE(tree_model), &iter,
+					ColumnIconName, columns[n],
+						-1);
+				++hiddencolumns;
+				continue;
+			}
+
+			coltype = gtk_tree_model_get_column_type(
+					tree_model, n + FirstDataColumn - 
+					hiddencolumns);
+			switch (coltype) {
+				case G_TYPE_STRING:
+					gtk_tree_store_set(
+						GTK_TREE_STORE(tree_model),
+						&iter,
+						n + FirstDataColumn - 
+						hiddencolumns, columns[n],
+						-1);
+					break;
+				default:
+					PIP_DEBUG("Unhandled column type.");
+			}
+		}
+		g_strfreev(columns);
+	}
+
+	pclose(infile);
+}
+
+void 
+widget_tree_refresh(variable *var)
+{
+	gchar        *act;
+	GtkTreeModel *tree_model;
+	
+	if (var == NULL || var->Attributes == NULL)
+		return;
+
+	PIP_DEBUG("Refreshing variable %p.", var);
+
+	g_return_if_fail(GTK_IS_TREE_VIEW(var->Widget));
+	tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(var->Widget));
+	g_return_if_fail(GTK_IS_TREE_STORE(tree_model));
+	/*
+	 * We drop all the lines here.
+	 */
+	gtk_tree_store_clear(GTK_TREE_STORE(tree_model));
+	/*
+	 * If the widget has an input tag (command, file, stc.) we load the
+	 * lines from there.
+	 */
+	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
+	while (act != NULL) {
+		gchar         *tmp;
+		gint           stock_column = -1;
+		gint           icon_column = -1;
+
+		tmp = attributeset_get_this_tagattr(var->Attributes, ATTR_INPUT, "stock_column");
+		if (tmp != NULL)
+			stock_column = atoi(tmp);
+		tmp = attributeset_get_this_tagattr(var->Attributes, ATTR_INPUT, "icon_column");
+		if (tmp != NULL)
+			icon_column = atoi(tmp);
+
+		if (input_is_shell_command(act))
+			fill_tree_view_by_command(var->Widget, tree_model, act + 8, stock_column, icon_column);
+		else
+			fill_tree_view_by_command(var->Widget, tree_model, act, stock_column, icon_column);
+		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
+	}
+	/*
+	 * If there are items, we load them too.
+	 */
+	fill_tree_model_by_items(tree_model, var);
+}
+
+
+
+
+void 
+widget_table_refresh(variable * var)
 {
 	char *act;
 	if (var == NULL || var->Attributes == NULL)
@@ -480,7 +699,8 @@ void widget_table_refresh(variable * var)
 		gtk_widget_set_sensitive(var->Widget, FALSE);
 }
 
-void widget_combo_refresh(variable * var)
+void 
+widget_combo_refresh(variable * var)
 {
 	if (var == NULL || var->Attributes == NULL)
 		return;
@@ -497,13 +717,15 @@ void widget_combo_refresh(variable * var)
 
 extern gchar *option_include_file;
 
-FILE *widget_opencommand(const char *command)
+FILE *
+widget_opencommand(
+		const char *command)
 {
 	char *the_line;
 	FILE *infile;
-#ifdef DEBUG
-	g_message("%s(%s)", __func__, command);
-#endif
+
+	PIP_DEBUG("Opening command: '%s'", command);
+
 	if (option_include_file != NULL) {
 		the_line = g_strdup_printf("source %s; %s", 
 				option_include_file, command);
@@ -678,7 +900,11 @@ void fill_checkbox_by_command(GtkWidget *checkbox, char *command)
 }
 
 
-void fill_table_by_command(GtkWidget * list, char *command)
+
+void 
+fill_table_by_command(
+	GtkWidget * list, 
+	char *command)
 {
 	FILE *infile;
 	char oneline[512];
@@ -969,3 +1195,4 @@ widget_get_type_from_pointer(GtkWidget *widget)
 		return WIDGET_BUTTON;
 	return 0;
 }
+
