@@ -16,12 +16,19 @@
 #include "variables.h"
 #include "automaton.h"
 
+typedef enum {
+	ColumnPixbuf,
+	ColumnIconName,
+	ColumnStockId,
+	FirstDataColumn
+};
 
 char *widget_get_text_value( GtkWidget *widget, int type );
 int widget_label_refresh( variable *var );
 int widget_edit_refresh( variable *var );
 void widget_entry_refresh( variable *var );
 void widget_list_refresh( variable *var );
+void widget_tree_refresh( variable *var );
 void widget_table_refresh( variable *var );
 void widget_combo_refresh( variable *var );
 FILE *widget_opencommand( const char *command );
